<?php
// $Id$

/**
 * @file
 * OM Maximenu Admin Utilities
 *
 * @author: Daniel Honrade http://drupal.org/user/351112
 *
 */
define('OM_MAXIMENU_PATH', drupal_get_path('module', 'om_maximenu')); 

drupal_add_css(OM_MAXIMENU_PATH . '/css/om_maximenu.css');
drupal_add_js(OM_MAXIMENU_PATH . '/js/om_maximenu.js'); 

$_om_maximenu_variable = array();

// om maximenu variable, js and skins loader
om_maximenu_loader();

/**
 * Loads styles and js
 *
 */
function om_maximenu_loader() {
  global $_om_maximenu_variable;
  
  // all menus settings
  $_om_maximenu_variable  = variable_get('om_maximenu', array());

  $maximenus = $_om_maximenu_variable;
  
  if (!empty($maximenus)) {
    foreach ($maximenus as $menu_key => $menu_content) {
  
      $skin          = $menu_content['skin'];
      $fade          = $menu_content['fade']; 
      $animated_bg   = isset($menu_content['animated_bg']) ? 1: 0; 
      $dock          = isset($menu_content['float_options']['dock']) ? 1: 0;
      $maximenu_name = om_string_name($menu_content['title']); 
      
      // add css per OM Maximenu     
      drupal_add_css(OM_MAXIMENU_PATH . '/skin/' . $skin . '/' . $skin . '.css');
      
      // add js per OM Maximenu
      if ($fade) om_maximenu_hover_js($fade, 'om-maximenu-' . $maximenu_name); 
      
      // jquery animated bg effect
      if ($animated_bg) om_maximenu_bg_slider($maximenu_name); 
      
      // jquery fisheye dock effect 
      if ($dock) om_maximenu_dock($dock, $maximenu_name);
    }
  }
}


/**
 * Load all OM Maximenu blocks
 * 
 * This will process all created OM Maximenu blocks
 *
 */
function om_maximenu_block_load($maximenu_delta = '') {
  global $_om_maximenu_variable;
  $out = array();
  
  $maximenu_delta = explode('-', $maximenu_delta);
  // ex. $maximenu_delta = 'om-maximenu-1';
  $delta = $maximenu_delta[2];
  // om delta always starts with 1
  if (isset($maximenus[$delta]) && ($delta != 0)) {
  
    $maximenus = $_om_maximenu_variable;
    
    if (!empty($maximenus) && ($maximenus[$delta]['output'] == 'block')) {
      $animated_bg    = isset($maximenus[$delta]['animated_bg']) ? 1: 0;
      $dock           = isset($maximenus[$delta]['float_options']['dock']) ? 1: 0;      
      $maximenu_name  = om_string_name($maximenus[$delta]['title']);       
      $out['title']   = check_plain($maximenus[$delta]['title']);
      $out['content'] = om_maximenu_links_render($maximenus[$delta], $maximenu_name, $animated_bg, $dock);
    }
  }
  return $out;  
}


/**
 * Load all OM Maximenu blocks
 * 
 * This will process the main menu and floating blocks
 *
 */
function om_maximenu_load() {
  global $_om_maximenu_variable;

  $maximenus = $_om_maximenu_variable;

  $out = array();
  foreach ($maximenus as $menu_key => $menu_content) {
    if ($menu_content['output'] != 'block') {   
      $animated_bg   = isset($menu_content['animated_bg']) ? 1: 0;
      $dock          = isset($menu_content['float_options']['dock']) ? 1: 0; 
      $maximenu_name = om_string_name($menu_content['title']); 
      //$out[$menu_key]['title'] = $menu_content['title'];
      $out[$menu_key]['output'] = $menu_content['output'];
      $out[$menu_key]['content'] = om_maximenu_links_render($maximenus[$menu_key], $maximenu_name, $animated_bg, $dock);     
    }
  }
  return $out;  
}


/**
 * OM Maximenu links rendering engine
 *
 */
function om_maximenu_links_render($links = array(), $maximenu_name = NULL, $animated_bg = 0, $dock = 0) {
  if (!isset($links['links'])) return NULL;
  global $user;
  //dsm($user);
  $user_id = $user->uid;  
  $user_roles = $user->roles;
  $count = 0;
  $total = count($links['links']);
  $skin = ' om-maximenu-' . om_string_name($links['skin']);
  $output = ' om-maximenu-' . om_string_name($links['output']);
  
  if ($links['output'] == 'block') {
    $output_options = ' om-maximenu-block' . om_string_name($links['block_options']['direction']);   
  }
  elseif ($links['output'] == 'float') {
    $output_options = ' om-maximenu-' . om_string_name($links['float_options']['position']);   
  }
  elseif ($links['output'] == 'main_menu') {
    // no options yet
  }
  ($dock) ? $dock_class = ' om-maximenu-dock': $dock_class = '';
  
  $out = '<div id="om-maximenu-' . $maximenu_name . '" class="om-maximenu' . $skin . $dock_class . $output . $output_options . '">';
  $out .= '<ul id="om-menu-' . $maximenu_name . '" class="om-menu">';  

  usort($links['links'], 'om_sort_by_weight'); 
  $attributes = array();
  foreach ($links['links'] as $key => $content) {
    // user permission to view the link
    // roles
    // 1 - anonymous
    // 2 - authenticated
    //dsm($content['roles']);
    $roles = $content['roles'];
    $permission = array();
    $anyone = 0;
    foreach ($roles as $role => $value) {
       // check roles
       if (($value != 0) && isset($user_roles[$role])) $permission[$role] = $user_roles[$role];
       // for all roles are unchecked
       if ($value != 0) $anyone++;       
    }
    // the link will be visible if the user has permission or
    // the links is for everybody or
    // the user is a super user
    if (!empty($permission) || ($anyone == 0) || ($user_id == 1)) {    
      $count++;
      //dsm($content);
      $class_id = om_string_name($content['link_title']);
                        
      // standard li classes
      ($count == 1) ? $classes = ' first': $classes = '';
      if ($count == $total) $classes .= ' last';  
      $classes .= ' leaf-' . $class_id;     
      if ($_GET['q'] == $content['path']) $classes .= ' active'; 
      if (empty($content['path']) && drupal_is_front_page()) {
        $classes .= ' active'; 
        $content['class'] .= ' active'; 
      }  
      
      // user role class
      foreach ($permission as $pkey => $pval) {
        $pval = om_string_name($pval);
        $classes .= ' leaf-role-' . $pval; 
      }        
      
      // title, icon options
      $link_title_option = $content['link_title_option'];
      $link_title = check_plain($content['link_title']);      
      $link_icon = !empty($content['path_icon']) ? '<img src="' . base_path() . $content['path_icon'] . '" alt="' . $link_title . '" />': '';
      
      if ($link_title_option == 'title_icon') {
        $link_option = $link_icon . ' ' . $link_title;
        $classes .= ' om-leaf-title-icon';
      }
      elseif ($link_title_option == 'icon') {
        $link_option = $link_icon;
        $classes .= ' om-leaf-icon';
      }
      else {
        $link_option = $link_title;
        $classes .= '';
      }
      
      // id link attributes
      if (!empty($content['id'])) $attributes['id'] = $content['id'];
      $span_id = (!empty($content['id'])) ? 'id="' . $content['id'] . '"': '';        

      // class link attributes      
      $attributes['class'] = 'om-link ' . $content['class'] . ' link-' . $class_id;
      
      if ($content['description_option'] == 'hover') {
        if (!empty($content['description'])) $attributes['title'] = check_plain($content['description']); 
        $span_title = (!empty($content['description'])) ? 'title="' . check_plain($content['description']) . '"': ''; 
        $link_subtitle = '';
      }
      else {
        $span_title = '';
        $link_subtitle = (!empty($content['description'])) ? '<span class="om-subtitle">' . check_plain($content['description']) . '</span>': ''; 
      } 
      
      // relationship
      if (!empty($content['rel'])) $attributes['rel'] = $content['rel'];
      
      // target
      if (!empty($content['target']) && ($content['target'] != '_self')) $attributes['target'] = $content['target'];
            
      $out .= '<li id="om-leaf-' . $key . '" class="om-leaf' . $classes . '">';    
      if (!empty($content['path'])) {
        // Drupal doesn't accept # as valid path
        //$out .= l($content['link_title'], $content['path'], array('attributes' => $attributes));
        $path = ($content['path'] == '<front>') ? '': $content['path'];
        $out .= '<a ' . drupal_attributes($attributes) . ' href="' . base_path() . $path . '">' . $link_option . $link_subtitle . '</a>';
      }
      else {
        // useful when you just want a button for getting the content to show and not actually linking to anything
        $out .= '<span ' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . $link_subtitle . '</span>';
      }
      
      $out .= om_maximenu_content_render($content['content']);    
      $out .= '</li>';
    }
  }
  $out .= '</ul>';  
  
  // for animated sliding background
  if ($animated_bg) $out .= '<div id="om-maximenu-slider-' . $maximenu_name . '" class="om-maximenu-slider"></div>';   
   
  // access on editing
  if (user_access('administer OM Maximenu')) $out .= '<div class="om-maximenu-edit"><a href="' . base_path() . 'admin/settings/om-maximenu?destination=' . $_GET['q'] . '">Maxedit</a></div>';    
  
  $out .= '</div>';  
  
  return $out;
}


/**
 * OM Maximenu content rendering engine
 *
 */
function om_maximenu_content_render($content = '') {

  if (!empty($content)) {
    // enclose the content with divs for efficient image border theming
    $out = '<div class="om-maximenu-content om-maximenu-content-nofade">';
    $out .= '<div class="om-maximenu-top"><div class="om-maximenu-top-left"></div><div class="om-maximenu-top-right"></div></div>';
    $out .= '<div class="om-maximenu-middle"><div class="om-maximenu-middle-left"><div class="om-maximenu-middle-right">';      
    $count = 0;
    usort($content, 'om_sort_by_weight'); 
    $total = count($content); 

    foreach ($content as $key => $prop) {
        $count++;
        ($count == 1) ? $classes = ' first': $classes = '';
        if ($count == $total) $classes .= ' last'; 
        
        $path        = ($prop['title_path'] == '<front>') ? '': $prop['title_path'];
        $title       = check_plain($prop['title']);
        $block_title = (!empty($prop['title_path'])) ? '<a href="' . base_path() . $path .'" title="' . $title . '">' . $title . '</a>': $title;      
        $block       = module_invoke($prop['module'], 'block', 'view', $prop['delta']); 
        
        $out .= '<div class="block block-' . $prop['module'] . ' block-' . $prop['module'] . '-id-' . $prop['delta'] . $classes . '">';            
        $out .= ($prop['title'] != '<none>') ? '<h3 class="title">' . $block_title . '</h3>': ''; 
        $out .= $block['content']; 
        $out .= '</div>';            
    }
    $out .= '<div class="clearfix"></div></div></div></div>';      
    $out .= '<div class="om-maximenu-bottom"><div class="om-maximenu-bottom-left"></div><div class="om-maximenu-bottom-right"></div></div>';    
    $out .= '<div class="om-maximenu-arrow"></div>';    
    $out .= '</div>'; 
    return $out;
  } 
}

/**
 * Sorting by weight
 *
 */
function om_sort_by_weight($a, $b) {
  return $a['weight'] - $b['weight'];
}

/**
 * Sorting by weight
 *
 */
function om_string_name($string = NULL, $upper = 'lower') {
  // converts space and underscore to dash
  $pattern = '/[ _]+/';
  $string = ($upper == 'upper') ? ucwords(preg_replace($pattern, ' ', $string)): strtolower(preg_replace($pattern, '-', $string));
  $string = preg_replace('/[^a-zA-Z0-9-]+/', '-', $string);
  return $string;
}

/**
 * OM Maximenu get styles from /skin directory
 * - reads directory names
 */
function om_maximenu_style_get() {
  $files = array();
  $dir = OM_MAXIMENU_PATH . '/skin/';
  $folders = scandir($dir);
  $excluded_files = array('.', '..', '.cvs', '.svn', '.git');

  foreach ($folders as $key => $val) {
    if (!in_array($val, $excluded_files)) (is_dir($dir . $val)) ? $files[$val] = om_string_name($val, 'upper'): '';     
  }
  return $files;
}


/**
 * Hover delay using hoverintent jquery plugin
 *
 */
function om_maximenu_hover_js($checked = 0, $menu = 'om-maximenu') {
  
  $add_js = "
    $(document).ready(function(){
      $('#" . $menu . " li.om-leaf .om-maximenu-content').removeClass('om-maximenu-content-nofade');
      //$('#" . $menu . " li.om-leaf').hover(omFadeIn,omFadeOut); //native jquery
      //$('#" . $menu . " li.om-leaf').hoverIntent(omFadeIn,omFadeOut); //jquery hoverintent
      $('#" . $menu . " li.om-leaf').hoverIntent({
        over: omFadeIn,
        timeout: 500,
        out: omFadeOut
      });
    });
    function omFadeIn(){ $('.om-maximenu-content', this).fadeIn();}
    function omFadeOut(){ $('.om-maximenu-content', this).fadeOut();} 
    ";
  if ($checked == 1) {
    drupal_add_js(OM_MAXIMENU_PATH . '/contrib/jquery.hoverIntent.minified.js'); 
    drupal_add_js($add_js, "inline");
  }
}


/**
 * Dock jquery plugin
 *
 */
function om_maximenu_dock($checked = 0, $menu = '') {
  
  $add_js = "
    $(document).ready(function(){
			$('#om-maximenu-" . $menu . "').Fisheye({
					maxWidth: 60,
					items: '.om-link',
					itemsText: '.om-subtitle',
					container: '#om-menu-" . $menu . "',
					itemWidth: 40,
					proximity: 80,
					alignment : 'left',
					valign: 'bottom',
					halign : 'center'
				})
		});
    ";
  if ($checked == 1) {
    drupal_add_js(OM_MAXIMENU_PATH . '/contrib/interface.js'); 
    drupal_add_js($add_js, "inline");
  }
}

/**
 * Animated background slider
 *
 */
function om_maximenu_bg_slider($menu = NULL) {

  $add_js = "
    $(document).ready(function(){
      om_menu_slider.init('om-menu-" . $menu . "','om-maximenu-slider-" . $menu . "');
    });   
    var om_menu_slider=function(){
	  var m,e,g,s,q,i; e=[]; q=8; i=8;
	  return{
		  init:function(j,k){
			  m=document.getElementById(j); e=m.getElementsByTagName('li');
			  var i,l,w,p; i=0; l=e.length;
			  for(i;i<l;i++){
				  var c,v; c=e[i]; v=c.value; if(v==1){s=c; w=c.offsetWidth; p=c.offsetLeft}
				  c.onmouseover=function(){om_menu_slider.mo(this)}; c.onmouseout=function(){om_menu_slider.mo(s)};
		  	}
			  g=document.getElementById(k); g.style.width=w+'px'; g.style.left=p+'px';
		  },
		  mo:function(d){
			  clearInterval(m.tm);
			  var el,ew; el=parseInt(d.offsetLeft); ew=parseInt(d.offsetWidth);
			  m.tm=setInterval(function(){om_menu_slider.mv(el,ew)},i);
		  },
		  mv:function(el,ew){
			  var l,w; l=parseInt(g.offsetLeft); w=parseInt(g.offsetWidth);
			  if(l!=el||w!=ew){
				  if(l!=el){var ld,lr,li; ld=(l>el)?-1:1; lr=Math.abs(el-l); li=(lr<q)?ld*lr:ld*q; g.style.left=(l+li)+'px'}
				  if(w!=ew){var wd,wr,wi; wd=(w>ew)?-1:1; wr=Math.abs(ew-w); wi=(wr<q)?wd*wr:wd*q; g.style.width=(w+wi)+'px'}
			  }else{clearInterval(m.tm)}
      }
    };
  }();
  ";
  drupal_add_js($add_js, "inline");  
}
    

/**
 * Admin - hide/show options
 *
 */
function om_maximenu_admin_js() {
  
  $add_js = "
    $(document).ready(function(){

      $('form#om-maximenu-admin select.om-maximenu-output').change(function() {

        var parent_wrapper = $(this).parent().parent().parent().attr('id');  
        var output_value = $(this).val(); 
        
        if(output_value == 'block') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').show();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').hide();          
        }
        if(output_value == 'float') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').show();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').hide();
        }     
        if(output_value == 'main_menu') {
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-float-options').hide();
          $('form#om-maximenu-admin fieldset#' + parent_wrapper + ' .om-maximenu-block-options').hide();
        } 
      });
    }); ";
  
  drupal_add_js($add_js, "inline");
}



